"""
 __init__.py : Function for resolutions of dependencies

 Copyright (c) 2009, Karel Slany    (slany AT fit.vutbr.cz)
                     Zdenek Vasicek (vasicek AT fit.vutbr.cz)
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of the organization nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
"""

import types
import interface

#Import modules
import mod_get_dnservs
import mod_soa_recs
import mod_mx_recs
import mod_a_recs
import mod_ns_recs
import mod_axfr
import mod_ptr_recs
import mod_mail_test
import mod_www_test
import mod_dnssec_check

def get_class_list(moduleobj):
    """Returns a list of classes derived from interface.TestModule"""
    classes = [] 
    for obj in moduleobj.__dict__.values():
       try:
          if interface.TestModule in obj.__bases__: classes.append(obj)
       except:
          pass
    return classes

def get_module_list(print_deps=False):
    """Calculates the dependency of imported test modules

       :param print_deps: if True, prints modules and their dependencies
       :returns: ordered list of module objects
    """
    list = {}

    thismod = __import__(__name__)
    for m in [val for val in thismod.__dict__.values() if isinstance(val, types.ModuleType)]:

        classes = get_class_list(m) #list of classes derived from interface.TestModule

        for mcls in classes:
           if not mcls in list:
               list[mcls] = []

           for m in [val for val in m.__dict__.values() if isinstance(val, types.ModuleType) and len(get_class_list(val))]:
               dmcls = m.Module
               if not dmcls in list:
                   list[dmcls] = []

               if not dmcls in list[mcls]:
                   list[mcls].append(dmcls)

    #print list
    #for k in list:
    #    print "modul:",str(k),[str(k) for k in list[k]]

    #Transitive closure    
    modif = True
    while modif:
        modif = False
        for i in list:
            for j in list[i]:
                for k in list[j]:
                    if not k in list[i]:
                        list[i].append(k)
                        modif = True

    #Circle test
    for m in list:
        if m in list[m]:
            raise Exception("Building of dependency list failed - circular reference detected ("+str(m)+')')

    #Build list of deps
    modlist = []
    if not len(list):
       return modlist

    for i in range(0,max(len(list[m]) for m in list)+1):
        for m in list:
            if len(list[m]) == i:
                modlist.append(m)


    #print list
    if print_deps:
       for k in modlist:
           print "module ",str(k),"requires:"
           if (not len(list[k])):
               print "        nothing"
           for d in list[k]:
               print "       ",str(d)
           print

    return modlist
