"""
 mod_www_test.py : Module, test whether a HTTP server is present

 Copyright (c) 2009, Karel Slany    (slany AT fit.vutbr.cz)
                     Zdenek Vasicek (vasicek AT fit.vutbr.cz)
                     Marek Vavrusa  (xvavru00 AT stud.fit.vutbr.cz)
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of the organization nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
"""

from interface import TestModule

import mod_a_recs

import socket

def has_http(ipstr):
    # open socket
    af = socket.AF_INET
    if ipstr.find(":") != -1:
        af = socket.AF_INET6
    sock = socket.socket(af, socket.SOCK_STREAM)
    sock.settimeout(1)
    try:
        sock.connect((ipstr, 80))
    except:
        return False, ""
        
    def rdln(sock):
        ch, line = "",""
        while (ch != "\n"):
            ch = sock.recv(1)
            line += ch
        return line.strip()
        
    # attempt to read
    data = ""
    server = ""
    has_http = False
    try:
        sock.send("GET / HTTP/1.0\n\n")
        data = sock.recv(4)
        if (data == "HTTP"):
            has_http = True
            rdln(sock)
            while True: #read HTTP headers
                line = rdln(sock)
                if not line: #empty line
                    break
                print line
                hdr = line.split(":",2)
                if hdr[0].lower() == "server":
                    server = hdr[1].strip()
    except:
        pass
    sock.close()
    return has_http, server


class Module(TestModule):
    "Check all A records for HTTP server presence"

    def __init__(self, args, modenv):
        TestModule.__init__(self,args,modenv,"Test http servers")
        self.a_env = self.get_module_modenv(mod_a_recs)

    def execute(self):

        res = {}

        # check option
        if ((not self.args.options.arecs)
            or (not self.args.options.web)):
            self.set_my_modenv(res)
            return True

        res["http"] = {}

        for servname in self.a_env["dnservs"].keys():

            # for ipv4 addresses
            if (not self.args.options.noipv4
                and ("ipv4" in self.a_env["dnservs"][servname].keys())):
                for addr in self.a_env["dnservs"][servname]["ipv4"].keys():

                    if ((not self.args.options.noipv4)
                        and ("A" in self.a_env["dnservs"][servname]["ipv4"][addr].keys())):

                        for aname, rec_list in self.a_env["dnservs"][servname]["ipv4"][addr]["A"].iteritems():
                            if (aname not in res["http"].keys()):
                                res["http"][aname] = {}
                                res["http"][aname]["ipv4"] = []
                                res["http"][aname]["ipv6"] = []
                            for record in rec_list:
                                if (record["ipv4"] not in res["http"][aname]["ipv4"]):
                                    self.verb(2, "adding machine", aname, record["ipv4"])
                                    res["http"][aname]["ipv4"].append(record["ipv4"])

                    if (self.args.options.ipv6
                        and ("AAAA" in self.a_env["dnservs"][servname]["ipv4"][addr].keys())):
                        
                        for aname, rec_list in self.a_env["dnservs"][servname]["ipv4"][addr]["AAAA"].iteritems():
                            if (aname not in res["http"].keys()):
                                res["http"][aname] = {}
                                res["http"][aname]["ipv4"] = []
                                res["http"][aname]["ipv6"] = []
                            for record in rec_list:
                                if (record["ipv6"] not in res["http"][aname]["ipv6"]):
                                    self.verb(2, "adding machine", aname, record["ipv6"])
                                    res["http"][aname]["ipv6"].append(record["ipv6"])

            # for ipv6 adresses
            if (self.args.options.resolver6
                and ("ipv6" in self.a_env["dnservs"][servname].keys())):
                for addr in self.a_env["dnservs"][servname]["ipv6"].keys():
                    print addr

                    if ((not self.args.options.noipv4)
                        and ("A" in self.a_env["dnservs"][servname]["ipv6"][addr].keys())):

                        for aname, rec_list in self.a_env["dnservs"][servname]["ipv6"][addr]["A"].iteritems():
                            if (aname not in res["http"].keys()):
                                res["http"][aname] = {}
                                res["http"][aname]["ipv4"] = []
                                res["http"][aname]["ipv6"] = []
                            for record in rec_list:
                                if (record["ipv4"] not in res["http"][aname]["ipv4"]):
                                    self.verb(2, "adding machine", aname, record["ipv4"])
                                    res["http"][aname]["ipv4"].append(record["ipv4"])

                    if (self.args.options.ipv6
                        and ("AAAA" in self.a_env["dnservs"][servname]["ipv6"][addr].keys())):

                        for aname, rec_list in self.a_env["dnservs"][servname]["ipv6"][addr]["AAAA"].iteritems():
                            if (aname not in res["http"].keys()):
                                res["http"][aname] = {}
                                res["http"][aname]["ipv4"] = []
                                res["http"][aname]["ipv6"] = []
                            for record in rec_list:
                                if (record["ipv6"] not in res["http"][aname]["ipv6"]):
                                    self.verb(2, "adding machine", aname, record["ipv6"])
                                    res["http"][aname]["ipv6"].append(record["ipv6"])

        # querry by a system default resolver
        resolver = self.ldns.ldns_resolver.new_frm_file(None)

        for name in res["http"].keys():
            # try to resolve additional IPs

            addr = resolver.get_addr_by_name(name, self.ldns.LDNS_RR_CLASS_IN, self.ldns.LDNS_RD)

            for addr_rr in addr.rrs():

                if (not self.args.options.noipv4):
                    if (addr_rr.get_type() == self.ldns.LDNS_RR_TYPE_A):
                        if (str(addr_rr.rdf(0)) not in res["http"][name]["ipv4"]):
                            res["http"][name]["ipv4"].append(str(addr_rr.rdf(0)))
                            self.verb(2, "adding resolved machine", name, str(addr_rr.rdf(0)))
                else:
                    res["http"][name]["ipv4"] = []

                if (self.args.options.ipv6):
                    if (addr_rr.get_type() == self.ldns.LDNS_RR_TYPE_AAAA):
                        if (str(addr_rr.rdf(0)) not in res["http"][name]["ipv6"]):
                            res["http"][name]["ipv6"].append(str(addr_rr.rdf(0)))
                            self.verb(2, "adding resolved machine", name, str(addr_rr.rdf(0)))
                else:
                    res["http"][name]["ipv6"] = []

        # desolve only domain name
        name = self.args.options.dname
        addr = resolver.get_addr_by_name(name, self.ldns.LDNS_RR_CLASS_IN, self.ldns.LDNS_RD)
        if (addr):
            res["http"][name] = {}
            res["http"][name]["ipv4"] = []
            res["http"][name]["ipv6"] = []

            for addr_rr in addr.rrs():

                if (not self.args.options.noipv4):
                    if (addr_rr.get_type() == self.ldns.LDNS_RR_TYPE_A):
                        if (str(addr_rr.rdf(0)) not in res["http"][name]["ipv4"]):
                            res["http"][name]["ipv4"].append(str(addr_rr.rdf(0)))
                            self.verb(2, "adding resolved machine", name, str(addr_rr.rdf(0)))

                if (self.args.options.ipv6):
                    if (addr_rr.get_type() == self.ldns.LDNS_RR_TYPE_AAAA):
                        if (str(addr_rr.rdf(0)) not in res["http"][name]["ipv6"]):
                            res["http"][name]["ipv6"].append(str(addr_rr.rdf(0)))
                            self.verb(2, "adding resolved machine", name, str(addr_rr.rdf(0)))
        self.verb(2, "")

        # compute symmary
        some_active = False
        for name, addrlst in res["http"].iteritems():
            if (not self.args.options.noipv4):
                for addr in addrlst["ipv4"]:
                    http, serv = has_http(addr)
                    self.verb(1, "Machine (%s %s) has active HTTP: " % (name, addr), http)
                    self.verb(2, "Machine (%s %s) HTTP server: " % (name, addr), serv)
                    some_active = some_active or http
            if (self.args.options.ipv6):
                for addr in addrlst["ipv6"]:
                    http, serv = has_http(addr)
                    self.verb(1, "Machine (%s %s) has active HTTP: " % (name, addr), http)
                    self.verb(2, "Machine (%s %s) HTTP server: " % (name, addr), serv)
                    some_active = some_active or http
        self.verb(1, "")

        # print summary
        if (some_active):
            self.verb(0, "Some machines have active HTTP: yes")
        else:
            self.verb(0, "Some machines have active HTTP: no")
        self.verb(0, "")
        

        self.set_my_modenv(res)
        return True
